/*
==========================================
Name:		F_Interrupt.c
Author:		POSPISIL Frantisek
Created:	12/2008
Chip:		ATmega8535
Frequency:	CPU 8MHz
Hardware:	vystupni rele
==========================================
*/

/* hlavickove soubory */
#include <avr/io.h>
#include <avr/interrupt.h>
#include "F_Interrupt.h"
#include "F_DS1302.h"
#include "F_HD44780.h"
#include "F_SetMenu.h"
#include "Favorit_01.h"

/* definice preruseni */
// ===========================================================================
ISR(INT1_vect) // podprogram preruseni INT1 - rotacni koder
{
  SREG  |= 0x80; // nastavi globalni preruseni
  if (PINC&0x01) // neni stisknut koder
  {
    if ((~PINC&0x02) && (ROTcoder<ROTmax)) ROTcoder++;
    if (( PINC&0x02) && (ROTcoder>0)) ROTcoder--;
  }
  Beep( 2, 1 );
  if(((Stav1&0x07)==0x03) && (Timer2<0x40)) Timer2 += 10; // bude cekat jeste 10 sekund navic
}

// ===========================================================================
ISR(INT0_vect) // podprogram preruseni INT0 - civka motoru ***********************************
// posledni upravy 4.2.2009
{
  unsigned int OtackyMer;

  TCCR0 = 0;  // vypnuty citac
  OtackyMer = (256 * Otacky1) + TCNT0;
  TCNT0 = 0;
  TCCR0 = 3;  // normalni rezim s preddelickou 64
  Otacky1 = 0;
  if(OtackyCount<8) OtackyArray[OtackyCount] = OtackyMer; // prvni skupina mereni
  else
  {
    if((OtackyCount&0x0F)==8)
    {
	  OtackyCount = 0xF0;
	  OtackyERR = 0x00;
	  Otacky = ((unsigned int)(OtackyArray[0] + OtackyArray[1] + OtackyArray[2] + OtackyArray[3] + OtackyArray[4] + OtackyArray[5] + OtackyArray[6] + OtackyArray[7]) / 8);
	  if(Otacky<470) Beep( 2, 5 ); // otacky jsou nad 8000 za minutu
	}
	if((OtackyMer > (Otacky-(Otacky/2))) && (OtackyMer < (Otacky+(Otacky/2))))
	{
	  OtackyArray[(OtackyCount&0x0F)] = OtackyMer;
	}
	else 
	{
	  OtackyERR++; // pricita chybu
	}
	if(OtackyERR==4) // je prilis mnoho chyb
	{
	  OtackyCount = 0x00; // jede se od zacatku
	  OtackyERR = 0x00;
	  Beep( 1, 5 );
    }
  }
  OtackyCount++;
  Teploty(Teplomer); 
  Teplomer++;
  if(Otacky>1000)
  {
	Teploty(Teplomer); 
	Teplomer++;
  }
  if(Otacky>3000)
  {
	Teploty(Teplomer); 
	Teplomer++;
	Teploty(Teplomer); 
	Teplomer++;
	Teploty(Teplomer); 
	Teplomer++;
  }
  if(Otacky>4000)
  {
	Teploty(Teplomer); 
	Teplomer++;
	Teploty(Teplomer); 
	Teplomer++;
	Teploty(Teplomer); 
	Teplomer++;
  }
  if(Teplomer>100) Teplomer = 0;
}

// ===========================================================================
ISR (TIMER0_OVF_vect) // podprogram preruseni 0 - preteceni casovace 0, pro mereni otacek
{
  if(Otacky1!=0xFF)
  {
    Otacky1++;
	if(Otacky1==150) 
	{
	  Otacky1 = 0xFF; // otacky jsou pod 100ot/sec
	  OtackyCount = 0;
	}
  }
}

/* definice funkci */
// ===========================================================================
void Svetla(void) // ridi chod svetel v preruseni podle dveri a zapalovani
{
  unsigned int AD_prev; // hodnota z AD prevodniku

  if(PINC&0x04) Stav1 &=~0x08;
  else Stav1 |= 0x08;	// stav dveri je ve 3 bitu
  if(PINA&0x40) Stav1 |= 0x80;
  else Stav1 &=~0x80;	// stav zapalovani je v 7 bitu
  switch(Stav1&0x0F) // * * * * * * * * * ridi stav osvetleni kabiny
  {
    case 0x09: // otevrene dvere, stav 1 - rozsvecovani
	{
	  if(OCR1A>50) OCR1A -= 25; // rozsviti v kabine, doba je 4 sekundy
	  else Stav1++; // je rozsviceno stav 2
	  break;
	}
	case 0x0A: // otevrene dvere, stav 2 - sviti
	{
	  OCR1A = 0; // je rozsviceno
	  break;
	}
	case 0x0B: // otevrene dvere, stav 3 - ceka na zhasnuti
	{
	  Stav1 -= 1; // znovu rozsviti stav 2
	  break;
	}
	case 0x0C: // otevrene dvere, stav 4 - zhasina
	{
	  Stav1 -= 3; // znovu rozsviti stav 1
	  break;
	}
	case 0x0D: // otevrene dvere, stav 5 - zhasnuto
	{
	  Stav1 -= 4; // bude se rozsvecovat stav 1
	  break;
	}
	case 0x01: // zavrene dvere, stav 1
	{
	  Stav1 += 3; // jde na zhasinani stav 4
	  break;
	}
	case 0x02: // zavrene dvere, stav 2
	{
	  if(Stav1&0x80) // bezi motor?
	  {
	    Timer2 = DS1302( 0xE3, 0x00 ); // * * * * * * * * * * * * * doba cekani pred zhasnutim bezi motor
		Stav1 += 1; // jde na cekani stav 3
	  }
	  else
	  {
	    Timer2 = DS1302( 0xE1, 0x00 ); // * * * * * * * * * * * * * doba cekani pred zhasnutim
		Stav1 += 1; // jde na cekani stav 3
	  }
	  break;
	}
	case 0x03: // zavrene dvere, stav 3
	{
	  if(Timer2==0) Stav1 += 1; // jde na zhasinani
	  break;
	}
	case 0x04: // zavrene dvere, stav 4
	{
	  if(OCR1A<10000)
	  {
	    if(Stav1&0x80) OCR1A += 20; // zhasinani 4 sec, bezi motor
		else OCR1A += 5; // zhasinani 10 sec
	  }
	  else Stav1 += 1; // je zhasnuto, stav 5
	  break;
	}
	case 0x05: // zavrene dvere, stav 5
	{
	  OCR1A = 10050; // je zhasnuto
	  if((Stav1&0x80)!=0x80) Stav1 -= 5;	 // bude pocitat k vypnuti
	  Timer2 = DS1302( 0xED, 0x00 );   // * * * * * * * * * * * * * doba cekani pred vypnutim stroje
	  break;
	}
	case 0x00: // zavrene dvere, stav 0
	{
	  if(Stav1&0x80) Stav1++;
	  if(Timer2==0)
	  {
	    SREG &= ~0x80; // vypne preruseni
	    while(OCR1B<10000)
		{
		  OCR1B++;
		  Wait_ms( 4 );
		}
	    PORTD &= 0x0F; // vypne stroj
		while(1)
		{
		  Wait_ms(100);
		}
	  }
	  break;
	}
	case 0x08: // otevrene dvere, vraci se z 0x00
	{
	  Stav1++;
	  break;
	}
    case 0x07: // zavrene dvere, stav 7 - rozsvecovani po vypnuti motoru
	{
	  if(OCR1A>50) OCR1A -= 50; // rozsviti v kabine, doba je 2 sekundy
	  else Stav1--; // je rozsviceno stav 6
	  break;
	}
	case 0x06: // zavrene dvere, stav 6 - rozsviceno po vypnuti motoru
	{
	  OCR1A = 0; // je rozsviceno
	  if(Stav1&0x80) Stav1 -=2; // nove spusteni motoru, zhasina
	  break;
	}
	case 0x0E: // otevrene dvere, stav 6 - rozsviceno po vypnuti motoru
	{
	  Stav1 -= 4; // jde na 0x0A sviti s otevrenymi dvermi
	  break;
	}
	default: break;
  }
  switch(Stav1&0xF0) // * * * * * * * * * ridi stav svetel
  {
    case 0x90: // svetla nesviti, zapalovani zapnuto
	{
	  Timer3 = DS1302( 0xE5, 0x00 ); // * * * * * * * * * * * * * doba cekani na obrysove
	  Stav1 += 0x10; // jde se cekat na obrysove
	  break;
	}
    case 0xA0: // ceka na rozsviceni obrysovych, zapalovani zapnuto
	{
	  if(Timer3==0)
	  {
	    AD_prev = ADCL + (ADCH * 0x100); // precte napeti z prevodniku
	    if((AD_prev>625) && (Otacky1!=0xFF)) // napeti je vetsi nez 12,5V
		{
		  Timer3 = DS1302( 0xE7, 0x00 ); // * * * * * * * * * * * * * doba cekani na tlumene
		  Stav1 += 0x10; // jde se cekat na tlumene
		  Set_Obrys(); // rozsviti obrysova
		}
		else 
		{
		  Timer3 = 2; // * * * * * * * * * * * * * doba cekani na napeti
		  Beep( 3, 10 );
		}
	  }
	  break;
	}
    case 0xB0: // ceka na rozsviceni tlumenych, zapalovani zapnuto
	{
	  AD_prev = ADCL + (ADCH * 0x100); // precte napeti z prevodniku
	  if((AD_prev<625) || (Otacky1==0xFF)) // napeti je vetsi nez 12,5V
	  {
	    Stav1 -= 0x10;
		Timer3 = DS1302( 0xE5, 0x00 ); // * * * * * * * * * * * * * doba cekani na obrysove
	    Clr_Obrys(); // zhasne obrysova
	  }
	  if(Timer3==0)
	  {
	    Set_Tlum(); // rozsviti tlumena
		Stav1 += 0x10; // svetla jsou rozsvicena
	  }
	  break;
	}
    case 0x20: // ceka na zhasnuti obrysovych, zapalovani vypnuto
	{
	  if(Timer3==0)
	  {
	    Clr_Obrys(); // zhasne obrysova
	    Clr_Tlum(); // zhasne tlumena
		Stav1 -= 0x10; // svetla nesviti
	  }
	  break;
	}
    case 0x30: // ceka na zhasnuti tlumenych, zapalovani vypnuto
	{
	  if(Timer3==0)
	  {
	    Clr_Tlum(); // zhasne tlumena
		Stav1 -= 0x10; // jde cekat na obrysova
		Timer3 = DS1302( 0xEB, 0x00 ); // * * * * * * * * * * * * * doba cekani na zhasnuti obrysovych
	  }
	  break;
	}
    case 0x40: // svetla sviti, zapalovani vypnuto
	{
	  Stav1 -= 0x10; // bude cekat na zhasnuti
	  Timer3 = DS1302( 0xE9, 0x00 ); // * * * * * * * * * * * * * doba cekani na zhasnuti tlumenych
	  Stav1 |= 0x07; // rozsviti v kabine
	  break;
	}
  }
}
