/*
==========================================
Name:		F_SetMenu.c
Author:		POSPISIL Frantisek
Created:	12/2008
Chip:		ATmega8535
Frequency:	CPU 8MHz
Hardware:	rotacni koder
==========================================
*/

/* hlavickove soubory */
#include <avr/io.h>
#include "F_SetMenu.h"
#include "F_HD44780.h"
#include "F_DS1302.h"

/* definice funkci */
// ===========================================================================
void Beep(unsigned char val, unsigned char tim) // pomoci casovace 2 vytvori ton
{
  if(Beep1==0)
  {
    TCCR2 = 0x78 | (5 - ((val-1)&0x03));
    OCR2 = 0x80; // PWM jede na 50%
    Beep1 = tim;
	DDRD |= 0b10000000;
  }
}

// ===========================================================================
unsigned char ROT_Key(void) // vraci 1, kdyz je stisknut rotacni koder
{
  if (PINC&0x01) return 0x00;
  else
  {
    while((PINC&0x01)==0x00)
	{
	  Wait_us(10);
	}
	if((Stav1&0x07)==0x03) Timer2 += 10; // bude cekat jeste 10 sekund navic
	return 0x01;
  }
}

// ===========================================================================
void SET_Sterac(void) // nastaveni pocatecniho casu sterace
{
  unsigned char i, u;

  i = DS1302( 0xFD, 0x00 );
  if(i>38) i = 13;
  ROTcoder	= i;
  ROTmax	= 38;
  LCD_Init();
  LCD_Position( 0, 0 );
  LCD_WriteArray( 12 ); // "Cas steracu: "
  u = 1;
  while(u)
  {
    LCD_Position( 0, 13 );
	LCD_PrDec( ROTcoder + 2 );
	LCD_WriteData( 's' );
	if ((Stav1&0x80)==0x80) u = 0; // vyskoci ze smycky, kdyz se pusti zapalovani
	if (ROT_Key()==0x01) u = 0;
  }
  i = DS1302( 0xFC, ROTcoder ); //  ulozeni nastaveneho casu
  Sterace = ROTcoder;
}

// ===========================================================================
void SET_Hodiny(void) // nastaveni hodin
{
  unsigned char i, u, x;

  ROTcoder	= 0;
  ROTmax	= 3;
  LCD_Init();
  LCD_WriteComm(0x0F); // blikajici kurzor
  LCD_Position( 0, 0 );
  LCD_WriteArray( 13 ); // "Nastaveni hodin:    "
  i = 1;
  while(i)
  {
    LCD_Position( 1, 0 );
	LCD_Print_Timer(); // napise cas z obvodu DS1302
	LCD_Position( 2, 0 );
	LCD_WriteArray( 14 ); // " Zpet "
	switch(ROTcoder)
	{
	  case 0: LCD_Position( 1, 1 ); break;
	  case 1: LCD_Position( 1, 4 ); break;
	  case 2: LCD_Position( 1, 7 ); break;
	  case 3: LCD_Position( 2, 0 ); break;
	}
	Wait_ms( 100 );
	if ((Stav1&0x80)==0x80) i = 0; // vyskoci ze smycky, kdyz se pusti zapalovani
	if(ROT_Key())
	{
	  if(ROTcoder==0x03) i = 0;
	  else
	  {
	    u = ROTcoder; // ulozi se ROTcoder
		LCD_WriteComm(0x0E); // vypne blikani kurzoru
		if (u==0) ROTmax = 23;
		else ROTmax = 59;
		x = DS1302( 0x05 - (u * 2), 0x00 ); // vyjme hodnotu hodin
		ROTcoder = ((x / 16) * 10) + (x&0x0F);
		x = 1;
		while(x)
		{
		  LCD_Position( 1, u * 3 );
		  LCD_PrDec(ROTcoder);
		  LCD_Position( 1, 1 + (u * 3));
		  Wait_ms( 100 );
		  if ((Stav1&0x80)==0x80) x = 0; // vyskoci ze smycky, kdyz se pusti zapalovani
		  if(ROT_Key()) x = 0;
		}
		x = ((ROTcoder / 10) * 16) + (ROTcoder % 10);
		x = DS1302( 0x04 - (u * 2), x ); // ulozi hodnotu hodin
		LCD_WriteComm(0x0F); // blikajici kurzor
		ROTcoder = u; // vyjme se ROTcoder
	  }
	}
  }
}

// ===========================================================================
void SET_Cekani(void) // nastaveni cekacich casu
{
  unsigned char i, u, x, v=0;

  ROTcoder	= 0;
  ROTmax	= 7;
  LCD_Init();
  LCD_WriteComm(0x0F); // blikajici kurzor
  i = 1;
  while(i)
  {
	if(ROTcoder<4) LCD_WriteArray4( 15 ); // vypise hlavicku
	if(ROTcoder>3) LCD_WriteArray4( 19 ); // vypise hlavicku
	for(u=0; u<4; u++)
	{
	  LCD_Position( u, 18 );
	  if(ROTcoder<4) v = DS1302( 0xE1 + (2 * u), 0x00 );
	  if(ROTcoder>3) v = DS1302( 0xE9 + (2 * u), 0x00 );
	  if(v>99)
	  { 
	    LCD_WriteData( 'x' );
	    LCD_WriteData( 'x' );
	  }	    
	  else LCD_PrDec( v );
	  if((ROTcoder>3)&&(u==0x03))
	  {
	    LCD_Position( 3, 18 );
	    LCD_WriteSpace ( 2 );
	  }
	}
	if(ROT_Key()==0x01)
	{
	  if(ROTcoder<0x07)
	  {
		ROTmax = 99;
		v = DS1302( 0xE1 + (2 * ROTcoder), 0x00 );
		if(v>99) v = 10;
		u = ROTcoder; // ulozi ROTcoder
		ROTcoder = v;
		x = 1;
		LCD_WriteComm(0x0E); // vypne blikani kurzoru
		while(x)
		{
		  LCD_Position( u&0x03, 18 );
		  LCD_PrDec( ROTcoder );
		  LCD_Position( u&0x03, 17 );
		  Wait_ms( 50 );
		  if ((Stav1&0x80)==0x80) x = 0; // vyskoci ze smycky, kdyz se pusti zapalovani
		  if (ROT_Key()==0x01) x = 0;
		}
		v = (unsigned char) ROTcoder;
		LCD_WriteComm(0x0F); // blikajici kurzor
		ROTcoder = u; // vyjme ROTcoder
		ROTmax = 7;
		v = DS1302( 0xE0 + (2 * ROTcoder), v );
	  }
	  else i = 0;
	}
	LCD_Position( ROTcoder&0x03, 0 );
	Wait_ms( 50 );
	if ((Stav1&0x80)==0x80) i = 0; // vyskoci ze smycky, kdyz se pusti zapalovani
  }
}

// ===========================================================================
void SET_Displej(void) // nastaveni jasu displeje
{
  unsigned char i, u;

  i = DS1302( 0xFF, 0x00 );
  if(i>100) i = 50;
  ROTcoder	= i;
  ROTmax	= 100;
  LCD_Init();
  LCD_Position( 0, 0 );
  LCD_WriteArray( 23 ); // "Jas displeje: "
  u = 1;
  while(u)
  {
    LCD_Position( 0, 14 );
	LCD_PrDec3( ROTcoder );
	LCD_WriteData( '%' );
	OCR1B = 100 * (100 - ROTcoder);
	if ((Stav1&0x80)==0x80) u = 0; // vyskoci ze smycky, kdyz se pusti zapalovani
	if (ROT_Key()==0x01) u = 0;
  }
  i = DS1302( 0xFE, ROTcoder ); //  ulozeni nastaveneho jasu
}
